#  (gH)   --  galg.pl  ;  TimeStamp (unix) : 31 Juillet 03 12:03
#
#
########################################################################################
#
#
#  /***************************************************************/
#  /*                                                             */
#  /*  galg.pl  -- (gH) 2001  :                                   */
#  /*                                                             */
#  /*     +================================================+      */
#  /*     +                                                +      */
#  /*     +  gestion d'un fichier algorithme               +      */
#  /*     +                                                +      */
#  /*     +================================================+      */
#  /*                                                             */
#  /*                                                             */
#  /*  syntaxe :                                                  */
#  /*     galg   [ options  ]   nom_de_fichier                    */
#  /*     (voir ci-dessous le dtail des options)                 */
#  /*                                                             */
#  /***************************************************************/
#  /*                                                             */
#  /*  Auteur                                                     */
#  /*                                                             */
#  /*     gilles.hunault@univ-angers.fr                           */
#  /*     email : gilles.hunault@univ-angers.fr                   */
#  /*                                                             */
#  /*     http://www.info.univ-angers.fr/pub/gh/                  */
#  /*                                                             */
#  /*  Documentation                                              */
#  /*                                                             */
#  /*     http://www.info.univ-angers.fr/pub/gh/Galg.htm          */
#  /*                                                             */
#  /***************************************************************/

#########################################################################################

# numro de version

$version = "2.93" ;

# Chargement des sous-programmes mis dans des modules
# Il est conseill de regrouper tous les modules personnels dans un mme
# rpertoire et de mettre le nom de ce rpertoire dans
# la variable d'environnement PERL_MACROS ce qui permet
# d'utiliser la ligne suivante pour les localiser

  use lib $ENV{"PERL_MACROS"} ; # /* rpertoire des modules perl personnels */


# utilisation des modules spcifiques  galg

  use galgfncs  ; # Bibliothque de fonctions utiles comme mot, sansCmt etc.

  use galganl   ; # Module d'analyse proprement dit des algorithmes
  use galgvars  ; # Module de gestion des liste de variables, modules et tableaux
  use galgerr   ; # Module de gestion des erreurs

  use galglang  ; # Module de gestion des langages
  use galgtrad  ; # Module principal de traduction des algorithmes

  use galgamng  ; # Module d'amnagement d'un fichier texte en un algorithme


# dfinition (globale) de la ligne de syntaxe
# pour le sous-programme d'aide

  $ligneSyntaxe = " galg [ -t ft | -a fa [ -o lang [ -x parms ] ] ] | -e | -f [lang] ]" ;

# le sous-programme Aide rappelle la syntaxe de galg et
# dtaille (un peu) les options.

# le sous-programme galgcpy fournit un copyright et
# donne la rfrence des pages web pour une documentation plus approfondie

sub Aide {

    print "\n" ;
    print " galg.pl, version $version (gH) : gestion de fichiers algorithmes \n\n" ;
    print " syntaxe : $ligneSyntaxe \n\n" ;
    print "     -t    traite un fichier texte pour en faire un algorithme\n\n" ;
    print "     -a    analyse un fichier algorithme\n\n" ;
    print "           -o  traduit, aprs analyse dans le langage pass en paramtre \n\n" ;
    print "           -x  excute, aprs traduction avec passage\n" ;
    print "               ventuel des paramtres restants.\n\n" ;
    print "     -e    affiche la liste des erreurs reconnues\n\n" ;
    print "     -l    affiche la liste des langages connus pour traduction\n" ;
    print "           avec la commande \[de compilation et\] d'excution\n\n" ;
    print "     -f    donne les fonctions algorithmiques reconnues par le langage choisi\n\n" ;
    print "\n" ;
    print " exemples :  galg -e \n" ;
    print "             galg -l \n" ;
    print "             galg -f rexx\n" ;
    print "             galg -t triRapide.txt\n" ;
    print "             galg -a prodMat.alg \n" ;
    print "             galg -a nbParfaits.alg -o rex\n" ;
    print "             galg -a simulation.alg -o java -x 10 \"ventes\" 3.15 \n" ;

    print "\n"  ; # on force une ligne vide  l'cran
    &galgcpy()  ; # rappel du copyright et des pages web pour la doc
    exit(0)     ; # sortie avec un code-retour normal

 } ; # fin sub Aide


#########################################################################################
##                                                                                      #
##      programme principal                                                             #
##                                                                                      #
#########################################################################################


$dbg_Galg =  0  ; # variable pour dbogage : 0 en principe, 1 pour affichage "verbeux"


#########################################################################################
#                                                                                       #
#       partie 1.0 : vrification des paramtres                                        #
#                                                                                       #
#########################################################################################


 if ($dbg_Galg==1) {
    print "\n    ===> ".(1+$#ARGV)." paramtres fournis  savoir : \n" ;
    print "\n" ;
    my $leparm ; # paramtre courant
    my $nbparm ; # nombre de paramtres
    foreach $leparm (@ARGV) {
       $nbparm++ ;
       print "     paramtre ".sprintf("%3d",$nbparm)."  : ".$ARGV[$nbparm-1]."\n" ;
    } ; # fin de pour chaque paramtre
    print "\n" ;
 } ; # fin de si pour debug

 if ($#ARGV==-1) { &Aide(0) ; } ;

 # toutes les options commencent par le symbole - donc si le premier
 # caractre du premier paramtre n'est pas "-" on n'a pas slectionn
 # d'option, c'est une erreur : on le dit et on s'arrete aprs
 # avoir rappel la syntaxe

 if (substr($ARGV[0],0,1) ne "-") {

    print  "\n" ;
    print  "            ERREUR : vous n'avez spcifi aucune option (une option commence par un tiret).\n" ;
    print  "            Tapez galg sans paramtre pour la liste des options.\n" ;
    &galgcpy() ; exit(0) ;

 } ;  # fin de si aucune option spcifie

 &galglang::initNomLangages()  ; # initialisation de la liste des langages

 # gestion des arguments simples de la ligne de commande :
 #    - liste des erreurs recenses (option -e)
 #    - liste des langages reconnus pour la traduction (option -l)
 #    - liste des fonctions algorithmiques reconnues pour chaque langage  (option -f)

 if ($ARGV[0] eq "-e")   { &galgerr::listeDesErreurs()               ; &galgcpy() ; exit(0) ; } ;
 if ($ARGV[0] eq "-l")   { &galglang::listeLangages()                ; &galgcpy() ; exit(0) ; } ;
 if ($ARGV[0] eq "-f")   { &galglang::fonctionsDuLangage($ARGV[1])   ; &galgcpy() ; exit(0) ; } ;

 # si on arrive ici, seules -t et -a sont des options valides donc si ce n'est
 # pas le cas, on le dit et on s'arrete

 if (($ARGV[0] ne "-t") and ($ARGV[0] ne "-a") )  {

    print  "\n" ;
    print  "            ERREUR : vous n'avez spcifi aucune option valide (-a ou -t).\n" ;
    print  "            Tapez galg sans paramtre pour la liste des options.\n" ;
    &galgcpy() ; exit(0) ;

 } ;  # fin de si aucune option valide choisie

 # affichage du titre

 print "\n galg (gH) 2001 ; Gestion d'ALGorithmes\n\n" ;


 # dbut vritable du programme

 $modeAnl    =  0        ; # variable pour grer l'option -a  (Analyse)
 $modeAmng   =  0        ; # variable pour grer l'option -t  (Amnagement)
 $modeTrad   =  0        ; # variable pour grer l'option -o  (Traduction)
 $modeExec   =  0        ; # variable pour grer l'option -x  (Excution)
 $modePostc  =  0        ; # variable pour grer le mode post-comparaison

 $nomLang    =  ""       ; # nom du langage si traduction
 $nbParmRst  =  0        ; # nombre de paramtres  transmettre si excution
 $parmRst    =  ""       ; # paramtres  transmettre si excution

 $codeRet    =  0        ; # code retour final de galg

 $main::nbErreurs  = 0   ; # nombre global d'erreurs

 # gestion de la variable $modeAmng pour le mode Amenagement : option -t

 if ($ARGV[0] eq "-t")   {

   $modeAmng = 1 ;

   # on vrifie qu'aprs -t il y a bien le nom du fichier  amnager
   # sinon on le dit et on quitte le programme

   if ($#ARGV==0) {
       print "    ERREUR : aprs l'option -t il faut donner le nom du fichier-texte  amliorer\n\n" ;
       &galgcpy() ;
       exit(-1) ;
   } ; # fin si pas de nom de fichier aprs -t

   # on vrifie qu'aprs le nom du fichier  amnager il n'y a pas d'autre paramtre
   # sinon on le dit et on quitte le programme

   if ($#ARGV>1) {
       print "    ERREUR : avec l'option -t il ne faut rien mettre aprs le nom du fichier-texte  amliorer\n\n" ;
       &galgcpy() ;
       exit(-1) ;
   } ; # fin si pas de nom de fichier aprs -t

 } ; # fin si le paramtre Un est -t


# gestion de la variable $modeAnl pour le mode Analyse : option -a

if ($ARGV[0] eq "-a")   {

   $modeAnl = 1 ;
   if ($#ARGV==0) {
       print "    Erreur : aprs l'option -a il faut donner le nom du fichier-algorithme  analyser \n\n" ;
       &galgcpy() ;
       exit(-1) ;
   } ; # fin si pas de nom de fichier aprs -a

} ; # fin si le paramtre Un est -a

# vrification du nom du fichier : un fichier texte  amnager ou un fichier
# algorithme doit avoir une extension (et donc une chaine aprs le symbole .)

 my $idFourni = $ARGV[1] ; # Nom du Fichier Au Dpart :

 if ($dbg_Galg==1) { print " le nom du fichier fourni est $idFourni \n" ; } ;

 if ((index($idFourni,".") == -1) or (substr($idFourni,-1,1) eq "." )) {
    print STDERR " Erreur : votre fichier n'a pas d'extension dtecte.\n" ;

    # si le nom du fichier se termine par un point, on l'enlve pour le message d'explication

    if (substr($idFourni,-1,1) eq "." ) { $idFourni = substr($idFourni,0,length($idFourni)-1) } ;
    print STDERR " Il devrait s'apppeler $idFourni.alg ou $idFourni.txt plutot que $idFourni tout court.\n" ;
    &galgcpy() ;
    exit(-1) ;
 } ; # fin de si pas d'extension au fichier algorithme

 # si l'extension n'est pas .txt poyur -t ou ;alg pour -a, on le dit et on s'arrte

 my $idp        = index($idFourni,".") ; # indice du point
 my $extFournie = substr($idFourni,$idp+1) ;
 my $errExt     = 0 ;

 if (  $modeAmng == 1 ) {
    if ( $extFournie ne "txt" ) {
        print STDERR " Erreur : votre fichier n'a pas la bonne extension.\n" ;
        print STDERR " ce devrait tre .txt plutot que .$extFournie\n" ;
        $errExt++ ;
    } ; # fin de si extension non txt
 } ; # fin de si mode Amnagement

 if (  $modeAnl == 1 ) {
    if ( $extFournie ne "alg" ) {
        print STDERR " Erreur : votre fichier n'a pas la bonne extension.\n" ;
        print STDERR " ce devrait tre .alg plutot que .$extFournie\n" ;
        $errExt++ ;
    } ; # fin de si extension non alg
 } ; # fin de si mode Analyse

 if ( $errExt > 0 ) {
     &galgcpy() ;
     exit(-1) ;
 } ; # finsi erreur sur extension

 #  partir du nom complet du fichier, on rcupre la
 # partie commune aux divers noms, nomme base des noms de fichier :

 $baseNomFichier = &nomFichierSansExtension($idFourni) ;
 if ($dbg_Galg==1) { print " la base des noms de fichiers est donc $baseNomFichier \n" ; } ;

###############################################################################################

# on continue  vrifier les paramtres :
# il reste l'option -o de traduction et l'option -x d'excution

# gestion de la variable $modeTrad pour le mode Traduction : option -o

if ($#ARGV>1) {
   if ($ARGV[2] eq "-o") {
      $modeTrad = 1 ;
      $nomLang  = $ARGV[3] ;
      if ($nomLang eq "") {
          print "\n    Erreur : aprs l'option -o il faut donner le nom du langage pour traduire.\n" ;
          print "    Utiliser galg -l pour la liste des langages reconnus pour la traduction.\n\n" ;
          &galgcpy() ;
          exit(-1) ;
     } ; # fin si pas de nom de fichier aprs -o
     if (length($tabLang{$nomLang})>0) {
         if ($dbg_Galg==1) { print "    ===> demande de Traduction en ".$tabLang{$nomLang}." \n\n" ; } ;
     } else {
         print "\n    ERREUR ! Langage $nomLang non reconnu.\n"  ;
         print "    Utiliser galg -l pour la liste des langages reconnus pour la traduction.\n\n" ;
         &galgcpy() ;
         exit(-1) ;
     } ; # fin de si langage connu
   } else {
         print "\n    ERREUR ! Aprs le nom du fichier, vous devez utiliser l'option -o.\n\n"  ;
         &galgcpy() ;
         exit(-1) ;
   } ; # fin de si il faut traduire
} ; # fin de si on a plus de 2 arguments en ligne de commande

# gestion de la variable $modeExec pour leu mode Excution : option -x

if ($#ARGV>3) {
   if ($ARGV[4] eq "-x") {
      if ($dbg_Galg==1) { print "    ===> mode Excution demand.\n\n" ; } ;
      $modeExec = 1 ;
      # rcupration de la liste (ventuellement vide) des paramtres restants
      my $indParm   = 5  ; # indice du paramtre courante
      while ($indParm <= $#ARGV) { $parmRst .= " $ARGV[$indParm]" ; $indParm++ ; $nbParmRst++ ; } ;
   } else {
         print "\n    ERREUR ! Aprs le nom du langage, pour excuter, vous devez utiliser l'option -x.\n\n"  ;
         &galgcpy() ;
         exit(-1) ;
   } ; # fin de si il faut traduire
} ; # fin de si on a plus de 4 arguments en ligne de commande


#########################################################################################
#                                                                                       #
#       partie 2.0 : amnagement ou analyse                                             #
#                                                                                       #
#########################################################################################

# Une fois les paramtres analyss, en fonction du mode de traitement,
# on amnage ou on analyse

$nomFichierAlg   = "$baseNomFichier.alg" ; # nom du fichier-algorithme

if ($modeAmng==1) { # amnagement

   if (uc($idFourni) eq  uc($nomFichierAlg)) {
        print "\n" ;
        print " Erreur ! Le fichier de dpart et le fichier d'arrive auraient le mme\n" ;
        print "\n Il serait bon de renommer le fichier de dpart $idFourni en $baseNomFichier.txt par exemple.\n" ;
        print "\n" ;
        &galgcpy() ;
        exit(-1) ;
   } ; # fin de si mmes noms de fichiers

   print " -t : Amnagement du fichier $idFourni en $nomFichierAlg \n" ;

   &galgamng::amenageTexte($idFourni,$nomFichierAlg) ;

   if ($dbg_Galg==1) { print "    ===> fin d'amnagement \n" ; } ;

   print "\n" ;
   print " vous pouvez consulter $nomFichierAlg \n" ;

} ; # fin de si on amnage

if ($modeAnl == 1) { # analyse

   print " -a : Analyse du fichier-algorithme $nomFichierAlg\n\n" ;

   # analyse du fichier-algorithme avec numrotation des lignes,
   # des instructions, de la profondeur...

   ($nomLst,$varLst,@lignesAlg) = &galganl::analyseAlgorithme($nomFichierAlg) ;

   if ($dbg_Galg==1) { print "    ===> fin d'analyse \n" ; } ;

 # rappel des fichiers construits

   print "\n" ;
   print " vous pouvez consulter les fichiers : \n\n" ;
   print "      $nomLst (listing numrot) \n" ;
   print "      $varLst (liste des variables et modules) \n\n" ;

   # S'il y a des erreurs, on le dit et on affiche les lignes incrimines
   # avant de continuer.

   if ($main::nbErreurs>0) {
       print " \n ATTENTION : il y a eu $main::nbErreurs" ;
       if ($main::nbErreurs==1) { print " erreur dtecte" ; } else  { print " erreurs dtectes" ; } ;
       print " dans votre algorithme  savoir \n\n" ;
       my $indErr = 0 ; # indice d'erreur
       for ($indErr=1;$indErr<=$main::nbErreurs;$indErr++) {
           print  $main::tabErr[$indErr]."\n" ;
       } ; # fin pour indErr
       $codeRet = -1 ;
   }  else { print " Algorithme sans erreur dtecte.\n\n"
   } ; # fin de si il y a eu des erreurs dtectes.

} ; # fin de si on analyse

#########################################################################################
#                                                                                       #
#       partie 3.0 : traduction et excution ventuelle                                 #
#                                                                                       #
#########################################################################################

 if ( ($modeTrad ==1) and ($main::nbErreurs==0) ) {

    print " -o : Traduction du fichier-algorithme $nomFichierAlg en $nomLang\n\n" ;

    # avant de traduire on regarde s'il y a un fichier de post-comparaison
    # avant de pouvoir inclure une ligne d'en-tete qui le signale

    $main::nomDuProgramme = $baseNomFichier.$main::extLang{$nomLang} ;

    my $fichierCorresp = $baseNomFichier.".tdc" ;
    if (-r $fichierCorresp) { $modePostc = 1 ; } ;

    &galgtrad::TradAlg($lstModules, $lstVars, $lstTableaux, $baseNomFichier, $nomLang,
                                      $modeExec, $modePostc,$fichierCorresp,@lignesAlg) ;

    if ($dbg_Galg==1) { print "    ===> fin de traduction\n" ; } ;

    # post-traduction automatique des modules commencant par _

    if ($dbg_Galg>=1) { print "\n    ===> dbut de post-traduction\n" ; } ;

    &galglang::postTraduction($main::nomDuProgramme,$nomLang) ;

    if ($dbg_Galg>=1) { print "    ===> fin   de post-traduction\n\n" ; } ;

    # s'il y a un fichier de post-comparaison, on remplace mot pour mot

    if ($modePostc == 1) {
       if ($dbg_Galg>=1) { print "    ===> dbut de post-comparaison\n" ; } ;
       &galglang::postComparaison($main::nomDuProgramme,$fichierCorresp,$nomLang) ;
       if ($dbg_Galg>=1) { print "    ===> fin   de post-comparaison\n\n" ; } ;

    } ; # fin de si pour post-comparaison

 } ; # fin de si il faut traduire

 if ( ($modeExec == 1) and ($main::nbErreurs==0) ) {

    print " -x : Excution du fichier produit : $main::nomDuProgramme\n\n" ;
    $cmd = $cmdLang{$nomLang}." $main::nomDuProgramme $parmRst" ;
    $ligcmd = "Excution de la commande $cmd" ;
    print " $ligcmd\n" ;
    print " ".("=" x length($ligcmd))."\n" ;

    my $retExec = system($cmd) ;

    print " ".("=" x length($ligcmd))."\n" ;
    if ($dbg_Galg==1) { print "    ===> fin d'excution\n" ; } ;
    if ($retExec != 0) {
       &galgcpy() ;
       exit($retExec) ;
    } ; # fin de si Excution incorrecte

 } ; # fin de si il faut excuter


#########################################################################################

# on finit par le rappel du copyright et le rappel des pages Web de documentation

&galgcpy() ; exit( $codeRet ) ;


#########################################################################################
##                                                                                      #
##      fin du programme principal                                                      #
##                                                                                      #
#########################################################################################


#########################################################################################


#########################################################################################
##                                                                                     ##
##                 fin du programme galg.pl                                            ##
##                                                                                     ##
#########################################################################################
